@@ -11,7 +11,7 @@ from logging import getLogger
 
 import pytz
 
-from .compat import (IS_BINARY, TO_UNICODE, IS_NUMERIC)
+from .compat import (PY2, IS_BINARY, TO_UNICODE, IS_NUMERIC)
 from .errorcode import (
     ER_NOT_SUPPORT_DATA_TYPE)
 from .errors import (ProgrammingError)
@@ -732,6 +732,8 @@ class SnowflakeConverter(object):
             return u'NULL'
         elif isinstance(value, bool):
             return u'TRUE' if value else u'FALSE'
+        elif PY2 and isinstance(value, long):
+            return TO_UNICODE(str(value))
         elif IS_NUMERIC(value):
             return TO_UNICODE(repr(value))
         elif IS_BINARY(value):
