@@ -15,20 +15,23 @@ from six import u
 
 from .chunk_downloader import (DEFAULT_CLIENT_RESULT_PREFETCH_SLOTS,
                                DEFAULT_CLIENT_RESULT_PREFETCH_THREADS)
-from .compat import (BASE_EXCEPTION_CLASS, TO_UNICODE)
-from .constants import (FIELD_NAME_TO_ID, FIELD_ID_TO_NAME)
+from .compat import (BASE_EXCEPTION_CLASS)
+from .constants import (
+    FIELD_NAME_TO_ID,
+    FIELD_ID_TO_NAME,
+)
 from .errorcode import (ER_UNSUPPORTED_METHOD,
                         ER_CURSOR_IS_CLOSED,
                         ER_FAILED_TO_REWRITE_MULTI_ROW_INSERT,
                         ER_NOT_POSITIVE_SIZE,
-                        ER_FAILED_PROCESSING_PYFORMAT,
                         ER_FAILED_TO_CONVERT_ROW_TO_PYTHON_TYPE,
-                        ER_INVALID_VALUE,
-                        ER_NOT_IMPLICITY_SNOWFLAKE_DATATYPE)
+                        ER_INVALID_VALUE)
 from .errors import (Error, ProgrammingError, NotSupportedError,
                      DatabaseError, InterfaceError)
 from .file_transfer_agent import (SnowflakeFileTransferAgent)
 from .sqlstate import (SQLSTATE_FEATURE_NOT_SUPPORTED)
+from .telemetry import (TelemetryData, TelemetryField)
+from .time_util import get_time_millis
 
 STATEMENT_TYPE_ID_DML = 0x3000
 STATEMENT_TYPE_ID_INSERT = STATEMENT_TYPE_ID_DML + 0x100
@@ -57,13 +60,15 @@ class SnowflakeCursor(object):
     PUT_SQL_RE = re.compile(u(r'^(?:/\*.*\*/\s*)*put\s+'), flags=re.IGNORECASE)
     GET_SQL_RE = re.compile(u(r'^(?:/\*.*\*/\s*)*get\s+'), flags=re.IGNORECASE)
     INSERT_SQL_RE = re.compile(u(r'^insert\s+into'), flags=re.IGNORECASE)
-    COMMENT_SQL_RE = re.compile(u"/\*.*\*/")
+    COMMENT_SQL_RE = re.compile(r"/\*.*\*/")
     INSERT_SQL_VALUES_RE = re.compile(u(r'.*VALUES\s*(\(.*\)).*'),
                                       re.IGNORECASE | re.MULTILINE | re.DOTALL)
     ALTER_SESSION_RE = re.compile(
         u(r'alter\s+session\s+set\s+(.*)=\'?([^\']+)\'?\s*;'),
         flags=re.IGNORECASE | re.MULTILINE | re.DOTALL)
 
+    LOG_MAX_QUERY_LENGTH = 80
+
     def __init__(self, connection):
         self._connection = connection
 
@@ -96,6 +101,8 @@ class SnowflakeCursor(object):
 
         self._lock_canceling = Lock()
 
+        self._first_chunk_time = None
+
         self.reset()
 
     def __del__(self):
@@ -400,9 +407,8 @@ class SnowflakeCursor(object):
                     self._client_result_prefetch_threads = kv[u'value']
                 if u'CLIENT_RESULT_PREFETCH_SLOTS' in kv[u'name']:
                     self._client_result_prefetch_slots = kv[u'value']
-            with self._connection._lock_converter:
-                self._connection.converter.set_parameters(
-                    ret[u'data'][u'parameters'])
+            self._connection._set_parameters(
+                ret, self._connection._session_parameters)
 
         self._sequence_counter = -1
         return ret
@@ -417,7 +423,8 @@ class SnowflakeCursor(object):
                 _is_internal=False,
                 _no_results=False,
                 _use_ijson=False,
-                _is_put_get=None):
+                _is_put_get=None,
+                _raise_put_get_error=False):
         u"""
         Executes a command/query
         """
@@ -439,7 +446,7 @@ class SnowflakeCursor(object):
         if self._connection.is_pyformat:
             # pyformat/format paramstyle
             # client side binding
-            processed_params = self.__process_params(params)
+            processed_params = self._connection._process_params(params, self)
             logger.debug(u'binding: %s with input=%s, processed=%s',
                          command,
                          params, processed_params)
@@ -452,23 +459,23 @@ class SnowflakeCursor(object):
             # qmark and numeric paramstyle
             # server side binding
             query = command
-            processed_params = self.__process_params_qmarks(params)
+            processed_params = self._connection._process_params_qmarks(
+                params, self)
 
-        if logger.getEffectiveLevel() <= logging.DEBUG:
-            logger.debug(
-                u'query: [%s]',
-                u' '.join(line.strip() for line in query.split(u'\n')))
         m = DESC_TABLE_RE.match(query)
         if m:
             query1 = u'describe table {0}'.format(m.group(1))
             if logger.getEffectiveLevel() <= logging.WARNING:
-                logger.warning(
+                logger.info(
                     u'query was rewritten: org=%s, new=%s',
                     u' '.join(line.strip() for line in query.split(u'\n')),
                     query1
                 )
             query = query1
 
+        if logger.getEffectiveLevel() <= logging.INFO:
+            logger.info(
+                u'query: [%s]', self._format_query_for_log(query))
         ret = self._execute_helper(
             query,
             timeout=timeout,
@@ -483,19 +490,21 @@ class SnowflakeCursor(object):
         self._sqlstate = ret[u'data'][
             u'sqlState'] if u'data' in ret and u'sqlState' in ret[
             u'data'] else None
+        self._first_chunk_time = get_time_millis()
+
+        # if server gives a send time, log the time it took to arrive
+        if u'data' in ret and u'sendResultTime' in ret[u'data']:
+            time_consume_first_result = self._first_chunk_time - ret[u'data'][
+                u'sendResultTime']
+            self._log_telemetry_job_data(
+                TelemetryField.TIME_CONSUME_FIRST_RESULT,
+                time_consume_first_result)
         logger.debug('sfqid: %s', self.sfqid)
 
+        logger.info('query execution done')
         if ret[u'success']:
             logger.debug(u'SUCCESS')
             data = ret[u'data']
-            if u'finalDatabaseName' in data:
-                self._connection._database = data[u'finalDatabaseName']
-            if u'finalSchemaName' in data:
-                self._connection._schema = data[u'finalSchemaName']
-            if u'finalWarehouseName' in data:
-                self._connection._warehouse = data[u'finalWarehouseName']
-            if u'finalRoleName' in data:
-                self._connection._role = data[u'finalRoleName']
 
             # logger.debug(ret)
             logger.debug(u"PUT OR GET: %s", self.is_file_transfer)
@@ -505,7 +514,8 @@ class SnowflakeCursor(object):
                     put_callback=_put_callback,
                     put_callback_output_stream=_put_callback_output_stream,
                     get_callback=_get_callback,
-                    get_callback_output_stream=_get_callback_output_stream)
+                    get_callback_output_stream=_get_callback_output_stream,
+                    raise_put_get_error=_raise_put_get_error)
                 sf_file_transfer_agent.execute()
                 data = sf_file_transfer_agent.result()
                 self._total_rowcount = len(data[u'rowset']) if \
@@ -542,10 +552,15 @@ class SnowflakeCursor(object):
                                        errvalue)
         return self
 
+    def _format_query_for_log(self, query):
+        ret = u' '.join(line.strip() for line in query.split(u'\n'))
+        return (ret if len(ret) < SnowflakeCursor.LOG_MAX_QUERY_LENGTH
+                else ret[0:SnowflakeCursor.LOG_MAX_QUERY_LENGTH] + '...')
+
     def _is_dml(self, data):
         return u'statementTypeId' in data \
                and int(data[u'statementTypeId']) in \
-                   STATEMENT_TYPE_ID_DML_SET
+               STATEMENT_TYPE_ID_DML_SET
 
     def chunk_info(self, data, use_ijson=False):
         is_dml = self._is_dml(data)
@@ -657,7 +672,7 @@ class SnowflakeCursor(object):
         u"""
         Executes a command/query with the given set of parameters sequentially.
         """
-        logger.info(u'executing many SQLs/commands')
+        logger.debug(u'executing many SQLs/commands')
         command = command.strip(u' \t\n\r') if command else None
 
         if len(seqparams) == 0:
@@ -689,7 +704,8 @@ class SnowflakeCursor(object):
                 values = []
                 for param in seqparams:
                     logger.debug(u'parameter: %s', param)
-                    values.append(fmt % self.__process_params(param))
+                    values.append(fmt % self._connection._process_params(
+                        param, self))
                 command = command.replace(fmt, u','.join(values), 1)
                 self.execute(command)
                 return self
@@ -726,6 +742,7 @@ class SnowflakeCursor(object):
         """
         Fetch one row
         """
+        is_done = False
         try:
             row = None
             self._total_row_index += 1
@@ -743,19 +760,35 @@ class SnowflakeCursor(object):
                     try:
                         row = next(self._current_chunk_row)
                     except StopIteration:
+                        is_done = True
                         raise IndexError
                 else:
                     if self._chunk_count > 0 and \
-                                    self._chunk_downloader is not None:
+                            self._chunk_downloader is not None:
                         self._chunk_downloader.terminate()
+                        self._log_telemetry_job_data(
+                            TelemetryField.TIME_DOWNLOADING_CHUNKS,
+                            self._chunk_downloader._total_millis_downloading_chunks)
+                        self._log_telemetry_job_data(
+                            TelemetryField.TIME_PARSING_CHUNKS,
+                            self._chunk_downloader._total_millis_parsing_chunks)
                     self._chunk_downloader = None
                     self._chunk_count = 0
                     self._current_chunk_row = iter(())
+                    is_done = True
+
             return self._row_to_python(row) if row is not None else None
 
         except IndexError:
             # returns None if the iteration is completed so that iter() stops
             return None
+        finally:
+            if is_done and self._first_chunk_time:
+                logger.info("fetching data done")
+                time_consume_last_result = get_time_millis() - self._first_chunk_time
+                self._log_telemetry_job_data(
+                    TelemetryField.TIME_CONSUME_LAST_RESULT,
+                    time_consume_last_result)
 
     def fetchmany(self, size=None):
         u"""
@@ -799,21 +832,21 @@ class SnowflakeCursor(object):
         u"""
         Not supporeted
         """
-        logger.info(u'nop')
+        logger.debug(u'nop')
         return None
 
     def setinputsizes(self, _):
         u"""
         Not supported
         """
-        logger.info(u'nop')
+        logger.debug(u'nop')
 
     def setoutputsize(self, _, column=None):
         u"""
         Not supported
         """
         del column
-        logger.info(u'nop')
+        logger.debug(u'nop')
 
     def scroll(self, value, mode=u'relative'):
         Error.errorhandler_wrapper(
@@ -835,7 +868,7 @@ class SnowflakeCursor(object):
         self._chunk_index = 0
 
         if hasattr(self, u'_chunk_count') and self._chunk_count > 0 and \
-                        self._chunk_downloader is not None:
+                self._chunk_downloader is not None:
             self._chunk_downloader.terminate()
 
         self._chunk_count = 0
@@ -849,124 +882,10 @@ class SnowflakeCursor(object):
 
     def __cancel_query(self, query):
         if self._sequence_counter >= 0 and not self.is_closed():
-            logger.debug(u'canceled : %s, %s',
-                         query, self._sequence_counter)
+            logger.debug(u'canceled. %s, request_id: %s',
+                         query, self._request_id)
             with self._lock_canceling:
-                self._connection._cancel_query(
-                    query,
-                    self._sequence_counter,
-                    self._request_id)
-
-    def __process_params_dict(self, params):
-        try:
-            to_snowflake = self._connection.converter.to_snowflake
-            escape = self._connection.converter.escape
-            quote = self._connection.converter.quote
-            res = {}
-            for k, v in params.items():
-                c = v
-                c = to_snowflake(c)
-                c = escape(c)
-                c = quote(c)
-                res[k] = c
-            logger.debug(u'parameters: %s', res)
-            return res
-        except Exception as e:
-            errorvalue = {
-                u'msg': u"Failed processing pyformat-parameters; {0}".format(
-                    e),
-                u'errno': ER_FAILED_PROCESSING_PYFORMAT}
-            Error.errorhandler_wrapper(
-                self.connection, self, ProgrammingError, errorvalue)
-
-    def __process_params(self, params):
-        if params is None:
-            return {}
-        if isinstance(params, dict):
-            return self.__process_params_dict(params)
-
-        if not isinstance(params, (tuple, list)):
-            params = [params, ]
-
-        try:
-            res = params
-            res = map(self._connection.converter.to_snowflake, res)
-            res = map(self._connection.converter.escape, res)
-            res = map(self._connection.converter.quote, res)
-            ret = tuple(res)
-            logger.debug(u'parameters: %s', ret)
-            return ret
-        except Exception as e:
-            errorvalue = {
-                u'msg': u"Failed processing pyformat-parameters; {0}".format(
-                    e),
-                u'errno': ER_FAILED_PROCESSING_PYFORMAT}
-            Error.errorhandler_wrapper(self.connection, self,
-                                       ProgrammingError,
-                                       errorvalue)
-
-    def __process_params_qmarks(self, params):
-        if not params:
-            return None
-        processed_params = {}
-        if not isinstance(params, (list, tuple)):
-            errorvalue = {
-                u'msg': u"Binding parameters must be a list: {0}".format(
-                    params
-                ),
-                u'errno': ER_FAILED_PROCESSING_PYFORMAT
-            }
-            Error.errorhandler_wrapper(self.connection, self,
-                                       ProgrammingError,
-                                       errorvalue)
-            return None
-        for idx, v in enumerate(params):
-            if isinstance(v, tuple):
-                if len(v) != 2:
-                    Error.errorhandler_wrapper(
-                        self.connection, self,
-                        ProgrammingError,
-                        {
-                            u'msg': u"Binding parameters must be a list "
-                                    u"where one element is a single value or "
-                                    u"a pair of Snowflake datatype and a value",
-                            u'errno': ER_FAILED_PROCESSING_PYFORMAT,
-                        }
-                    )
-                    return None
-                processed_params[TO_UNICODE(idx + 1)] = {
-                    'type': v[0],
-                    'value': self._connection.converter.to_snowflake_bindings(
-                        v[0], v[1])}
-            else:
-                snowflake_type = self._connection.converter.snowflake_type(v)
-                if snowflake_type is None:
-                    Error.errorhandler_wrapper(
-                        self.connection, self,
-                        ProgrammingError,
-                        {
-                            u'msg': u"Python data type [{0}] cannot be "
-                                    u"automatically mapped to Snowflake data "
-                                    u"type. Specify the snowflake data type "
-                                    u"explicitly.".format(
-                                v.__class__.__name__.lower()),
-                            u'errno': ER_NOT_IMPLICITY_SNOWFLAKE_DATATYPE
-                        }
-                    )
-                    return None
-                if isinstance(v, list):
-                    vv = [self._connection.converter.to_snowflake_bindings(
-                        snowflake_type, v0) for v0 in v]
-                else:
-                    vv = self._connection.converter.to_snowflake_bindings(
-                        snowflake_type, v)
-                processed_params[TO_UNICODE(idx + 1)] = {
-                    'type': snowflake_type,
-                    'value': vv}
-        if logger.getEffectiveLevel() <= logging.DEBUG:
-            for k, v in processed_params.items():
-                logger.debug("idx: %s, type: %s", k, v.get('type'))
-        return processed_params
+                self._connection._cancel_query(query, self._request_id)
 
     def _row_to_python(self, row):
         """
@@ -999,6 +918,23 @@ class SnowflakeCursor(object):
             idx += 1
         return tuple(row)
 
+    def _log_telemetry_job_data(self, telemetry_field, value):
+        u"""
+        Builds an instance of TelemetryData with the given field and logs it
+        """
+        obj = {
+            'type': telemetry_field,
+            'query_id': self._sfqid,
+            'value': int(value)
+        }
+        ts = get_time_millis()
+        try:
+            self._connection._log_telemetry(TelemetryData(obj, ts))
+        except AttributeError:
+            logger.warning(
+                "Cursor failed to log to telemetry. Connection object may be None.",
+                exc_info=True)
+
     def __enter__(self):
         """
         context manager
