@@ -16,7 +16,7 @@ ER_IDP_CONNECTION_ERROR = 250007
 ER_INCORRECT_DESTINATION = 250008
 ER_UNABLE_TO_OPEN_BROWSER = 250009
 ER_UNABLE_TO_START_WEBSERVER = 250010
-ER_INVALID_CERTIFICATE = 250011
+ER_INVALID_CERTIFICATE = 250011  # not used but keep here to reserve errno
 
 # connection
 ER_NO_ACCOUNT_NAME = 251001
@@ -27,6 +27,7 @@ ER_NO_USER = 251005
 ER_NO_PASSWORD = 251006
 ER_INVALID_VALUE = 251007
 ER_INVALID_PRIVATE_KEY = 251008
+ER_NO_HOSTNAME_FOUND = 251009
 
 # cursor
 ER_FAILED_TO_REWRITE_MULTI_ROW_INSERT = 252001
@@ -65,6 +66,7 @@ ER_OCSP_FAILED_TO_CONNECT_HOST = 254005
 ER_OPENSSL_IS_NOT_ACCESSIBLE = 254006
 ER_INVALID_OCSP_RESPONSE = 254007
 ER_CA_CERTIFICATE_NOT_FOUND = 254008
+ER_SERVER_CERTIFICATE_UNKNOWN = 254009
 
 # converter
 ER_NOT_SUPPORT_DATA_TYPE = 255001
