@@ -239,6 +239,17 @@ class BadGatewayError(Error):
             sfqid=kwargs.get('sfqid'))
 
 
+class MethodNotAllowed(Error):
+    u"""Exception for 405 HTTP error for retry"""
+
+    def __init__(self, **kwargs):
+        Error.__init__(
+            self, msg=kwargs.get('msg') or u'HTTP 405: Method not allowed',
+            errno=kwargs.get('errno'),
+            sqlstate=kwargs.get('sqlstate'),
+            sfqid=kwargs.get('sfqid'))
+
+
 class OtherHTTPRetryableError(Error):
     """
     Exception for other HTTP error for retry
@@ -251,16 +262,3 @@ class OtherHTTPRetryableError(Error):
             errno=kwargs.get('errno'),
             sqlstate=kwargs.get('sqlstate'),
             sfqid=kwargs.get('sfqid'))
-
-
-ER_MSG_FAILED_TO_VALIDATE_SSL_CERTIFICATE = (
-    u"Failed to validate SSL certificate. The potential cause is you are "
-    u"behind a SSL proxy that doesn't bypass the certificate. Call "
-    u"snowflake.connector.connect() method with probe_connection option to "
-    u"print out the certificate chain. URL: {}")
-
-ER_MSG_FAILED_TO_VALIDATE_SSL_CERTIFICATE_SNOWSQL = (
-    u"Failed to validate SSL certificate. The potential cause is you are "
-    u"behind a SSL proxy that doesn't bypass the certificate. Specify "
-    u"--probe-connection option to print out the certificate chain. "
-    u"URL: {}")
